---------------------------------------------------------------------------
--                                                                       --
--  ast2dot is an ASIS application that outputs the AST in dot format.   --
--                          http://cfg.w3x.org                           --
--                                                                       --
---------------------------------------------------------------------------
--                                                                       --
--        Copyright (c) 2006,2009 Raul Fechete, Georg Kienesberger       --
--           Copyright (c) 2000, Free Software Foundation, Inc.          --
--                                                                       --
---------------------------------------------------------------------------
-- This file is part of ast2dot, and is based on the ASIS Application    --
-- Templates (asis_application_driver_1.adb). Date of change:            --
-- 20 December 2006                                                      --
--                                                                       --
-- ast2dot is free software; you can redistribute it and/or modify       --
-- it under the terms of the GNU General Public License as published by  --
-- the Free Software Foundation; either version 2 of the License, or     --
-- (at your option) any later version.                                   --
--                                                                       --
-- ast2dot is distributed in the hope that it will be useful,            --
-- but WITHOUT ANY WARRANTY; without even the implied warranty of        --
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         --
-- GNU General Public License for more details.                          --
--                                                                       --
-- You should have received a copy of the GNU General Public License     --
-- along with ast2dot; if not, write to the Free Software                --
-- Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 --
-- USA                                                                   --
--                                                                       --
---------------------------------------------------------------------------

with Ada.Exceptions;
with Ada.Wide_Text_IO;
with Ada.Text_IO; use Ada.Text_IO;
with Ada.Characters.Handling;
with Ada.Command_Line; use Ada.Command_Line;
with Ada.Strings.Unbounded; use Ada.Strings.Unbounded;
with Ada.Directories; -- Ada 2005, but works (at least with my GNAT) - gk

with Asis;
with Asis.Ada_Environments;
with Asis.Implementation;
with Asis.Exceptions;
with Asis.Errors;

with Context_Processing;
with Options;

procedure ast2dot is

   My_Context            : Asis.Context;
   My_Context_Name       : Wide_String := Asis.Ada_Environments.Default_Name;
   My_Context_Parameters : Wide_String := Asis.Ada_Environments.Default_Parameters;
   Initialization_Parameters : Wide_String := "";
   Finalization_Parameters   : Wide_String := "";

   procedure Usage is
   begin
      Put_Line(Standard_Error, "ast2dot version " & Options.Version & ", Copyright (c) 2006,2009 Raul Fechete, Georg Kienesberger");
      Put_Line(Standard_Error, "ast2dot uses the ASIS Application Templates Copyright (c) 2000, Free Software");
      Put_Line(Standard_Error, "Foundation, Inc.");
      Put_Line(Standard_Error, "ast2dot comes with ABSOLUTELY NO WARRANTY; for details type `" & Command_Name & " -l'.");
      Put_Line(Standard_Error, "This is free software, and you are welcome to redistribute it under certain conditions;");
      Put_Line(Standard_Error, "type `" & Command_Name & " -l' for details.");
      Put_Line(Standard_Error, "");
      Put_Line(Standard_Error, "");
      Put_Line(Standard_Error, "Usage: " & Command_Name & " [-o DIRECTORY [-f] | -l]");
      Put_Line(Standard_Error, "Process all adt files in the current directory and write the dot");
      Put_Line(Standard_Error, "contents to stdout. Note that every digraph has to be in a separate");
      Put_Line(Standard_Error, "dot file.");
      Put_Line(Standard_Error, "");
      Put_Line(Standard_Error, "Options:");
      Put_Line(Standard_Error, " -o     (create DIRECTORY) and save a dot file for every graph there");
      Put_Line(Standard_Error, " -f     force DIRECTORY creation. If a directory with same name already");
      Put_Line(Standard_Error, "        exists, it will be deleted.");
      Put_Line(Standard_Error, " -l     show license details");

   end Usage;

   procedure ShowGPL is
   begin
      Put_Line("                    GNU GENERAL PUBLIC LICENSE");
      Put_Line("                       Version 2, June 1991");
      Put_Line("");
      Put_Line(" Copyright (C) 1989, 1991 Free Software Foundation, Inc.,");
      Put_Line(" 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA");
      Put_Line(" Everyone is permitted to copy and distribute verbatim copies");
      Put_Line(" of this license document, but changing it is not allowed.");
      Put_Line("");
      Put_Line("                            Preamble");
      Put_Line("");
      Put_Line("  The licenses for most software are designed to take away your");
      Put_Line("freedom to share and change it.  By contrast, the GNU General Public");
      Put_Line("License is intended to guarantee your freedom to share and change free");
      Put_Line("software--to make sure the software is free for all its users.  This");
      Put_Line("General Public License applies to most of the Free Software");
      Put_Line("Foundation's software and to any other program whose authors commit to");
      Put_Line("using it.  (Some other Free Software Foundation software is covered by");
      Put_Line("the GNU Lesser General Public License instead.)  You can apply it to");
      Put_Line("your programs, too.");
      Put_Line("");
      Put_Line("  When we speak of free software, we are referring to freedom, not");
      Put_Line("price.  Our General Public Licenses are designed to make sure that you");
      Put_Line("have the freedom to distribute copies of free software (and charge for");
      Put_Line("this service if you wish), that you receive source code or can get it");
      Put_Line("if you want it, that you can change the software or use pieces of it");
      Put_Line("in new free programs; and that you know you can do these things.");
      Put_Line("");
      Put_Line("  To protect your rights, we need to make restrictions that forbid");
      Put_Line("anyone to deny you these rights or to ask you to surrender the rights.");
      Put_Line("These restrictions translate to certain responsibilities for you if you");
      Put_Line("distribute copies of the software, or if you modify it.");
      Put_Line("");
      Put_Line("  For example, if you distribute copies of such a program, whether");
      Put_Line("gratis or for a fee, you must give the recipients all the rights that");
      Put_Line("you have.  You must make sure that they, too, receive or can get the");
      Put_Line("source code.  And you must show them these terms so they know their");
      Put_Line("rights.");
      Put_Line("");
      Put_Line("  We protect your rights with two steps: (1) copyright the software, and");
      Put_Line("(2) offer you this license which gives you legal permission to copy,");
      Put_Line("distribute and/or modify the software.");
      Put_Line("");
      Put_Line("  Also, for each author's protection and ours, we want to make certain");
      Put_Line("that everyone understands that there is no warranty for this free");
      Put_Line("software.  If the software is modified by someone else and passed on, we");
      Put_Line("want its recipients to know that what they have is not the original, so");
      Put_Line("that any problems introduced by others will not reflect on the original");
      Put_Line("authors' reputations.");
      Put_Line("");
      Put_Line("  Finally, any free program is threatened constantly by software");
      Put_Line("patents.  We wish to avoid the danger that redistributors of a free");
      Put_Line("program will individually obtain patent licenses, in effect making the");
      Put_Line("program proprietary.  To prevent this, we have made it clear that any");
      Put_Line("patent must be licensed for everyone's free use or not licensed at all.");
      Put_Line("");
      Put_Line("  The precise terms and conditions for copying, distribution and");
      Put_Line("modification follow.");
      Put_Line("");
      Put_Line("                    GNU GENERAL PUBLIC LICENSE");
      Put_Line("   TERMS AND CONDITIONS FOR COPYING, DISTRIBUTION AND MODIFICATION");
      Put_Line("");
      Put_Line("  0. This License applies to any program or other work which contains");
      Put_Line("a notice placed by the copyright holder saying it may be distributed");
      Put_Line("under the terms of this General Public License.  The ""Program"", below,");
      Put_Line("refers to any such program or work, and a ""work based on the Program""");
      Put_Line("means either the Program or any derivative work under copyright law:");
      Put_Line("that is to say, a work containing the Program or a portion of it,");
      Put_Line("either verbatim or with modifications and/or translated into another");
      Put_Line("language.  (Hereinafter, translation is included without limitation in");
      Put_Line("the term ""modification"".)  Each licensee is addressed as ""you"".");
      Put_Line("");
      Put_Line("Activities other than copying, distribution and modification are not");
      Put_Line("covered by this License; they are outside its scope.  The act of");
      Put_Line("running the Program is not restricted, and the output from the Program");
      Put_Line("is covered only if its contents constitute a work based on the");
      Put_Line("Program (independent of having been made by running the Program).");
      Put_Line("Whether that is true depends on what the Program does.");
      Put_Line("");
      Put_Line("  1. You may copy and distribute verbatim copies of the Program's");
      Put_Line("source code as you receive it, in any medium, provided that you");
      Put_Line("conspicuously and appropriately publish on each copy an appropriate");
      Put_Line("copyright notice and disclaimer of warranty; keep intact all the");
      Put_Line("notices that refer to this License and to the absence of any warranty;");
      Put_Line("and give any other recipients of the Program a copy of this License");
      Put_Line("along with the Program.");
      Put_Line("");
      Put_Line("You may charge a fee for the physical act of transferring a copy, and");
      Put_Line("you may at your option offer warranty protection in exchange for a fee.");
      Put_Line("");
      Put_Line("  2. You may modify your copy or copies of the Program or any portion");
      Put_Line("of it, thus forming a work based on the Program, and copy and");
      Put_Line("distribute such modifications or work under the terms of Section 1");
      Put_Line("above, provided that you also meet all of these conditions:");
      Put_Line("");
      Put_Line("    a) You must cause the modified files to carry prominent notices");
      Put_Line("    stating that you changed the files and the date of any change.");
      Put_Line("");
      Put_Line("    b) You must cause any work that you distribute or publish, that in");
      Put_Line("    whole or in part contains or is derived from the Program or any");
      Put_Line("    part thereof, to be licensed as a whole at no charge to all third");
      Put_Line("    parties under the terms of this License.");
      Put_Line("");
      Put_Line("    c) If the modified program normally reads commands interactively");
      Put_Line("    when run, you must cause it, when started running for such");
      Put_Line("    interactive use in the most ordinary way, to print or display an");
      Put_Line("    announcement including an appropriate copyright notice and a");
      Put_Line("    notice that there is no warranty (or else, saying that you provide");
      Put_Line("    a warranty) and that users may redistribute the program under");
      Put_Line("    these conditions, and telling the user how to view a copy of this");
      Put_Line("    License.  (Exception: if the Program itself is interactive but");
      Put_Line("    does not normally print such an announcement, your work based on");
      Put_Line("    the Program is not required to print an announcement.)");
      Put_Line("");
      Put_Line("These requirements apply to the modified work as a whole.  If");
      Put_Line("identifiable sections of that work are not derived from the Program,");
      Put_Line("and can be reasonably considered independent and separate works in");
      Put_Line("themselves, then this License, and its terms, do not apply to those");
      Put_Line("sections when you distribute them as separate works.  But when you");
      Put_Line("distribute the same sections as part of a whole which is a work based");
      Put_Line("on the Program, the distribution of the whole must be on the terms of");
      Put_Line("this License, whose permissions for other licensees extend to the");
      Put_Line("entire whole, and thus to each and every part regardless of who wrote it.");
      Put_Line("");
      Put_Line("Thus, it is not the intent of this section to claim rights or contest");
      Put_Line("your rights to work written entirely by you; rather, the intent is to");
      Put_Line("exercise the right to control the distribution of derivative or");
      Put_Line("collective works based on the Program.");
      Put_Line("");
      Put_Line("In addition, mere aggregation of another work not based on the Program");
      Put_Line("with the Program (or with a work based on the Program) on a volume of");
      Put_Line("a storage or distribution medium does not bring the other work under");
      Put_Line("the scope of this License.");
      Put_Line("");
      Put_Line("  3. You may copy and distribute the Program (or a work based on it,");
      Put_Line("under Section 2) in object code or executable form under the terms of");
      Put_Line("Sections 1 and 2 above provided that you also do one of the following:");
      Put_Line("");
      Put_Line("    a) Accompany it with the complete corresponding machine-readable");
      Put_Line("    source code, which must be distributed under the terms of Sections");
      Put_Line("    1 and 2 above on a medium customarily used for software interchange; or,");
      Put_Line("");
      Put_Line("    b) Accompany it with a written offer, valid for at least three");
      Put_Line("    years, to give any third party, for a charge no more than your");
      Put_Line("    cost of physically performing source distribution, a complete");
      Put_Line("    machine-readable copy of the corresponding source code, to be");
      Put_Line("    distributed under the terms of Sections 1 and 2 above on a medium");
      Put_Line("    customarily used for software interchange; or,");
      Put_Line("");
      Put_Line("    c) Accompany it with the information you received as to the offer");
      Put_Line("    to distribute corresponding source code.  (This alternative is");
      Put_Line("    allowed only for noncommercial distribution and only if you");
      Put_Line("    received the program in object code or executable form with such");
      Put_Line("    an offer, in accord with Subsection b above.)");
      Put_Line("");
      Put_Line("The source code for a work means the preferred form of the work for");
      Put_Line("making modifications to it.  For an executable work, complete source");
      Put_Line("code means all the source code for all modules it contains, plus any");
      Put_Line("associated interface definition files, plus the scripts used to");
      Put_Line("control compilation and installation of the executable.  However, as a");
      Put_Line("special exception, the source code distributed need not include");
      Put_Line("anything that is normally distributed (in either source or binary");
      Put_Line("form) with the major components (compiler, kernel, and so on) of the");
      Put_Line("operating system on which the executable runs, unless that component");
      Put_Line("itself accompanies the executable.");
      Put_Line("");
      Put_Line("If distribution of executable or object code is made by offering");
      Put_Line("access to copy from a designated place, then offering equivalent");
      Put_Line("access to copy the source code from the same place counts as");
      Put_Line("distribution of the source code, even though third parties are not");
      Put_Line("compelled to copy the source along with the object code.");
      Put_Line("");
      Put_Line("  4. You may not copy, modify, sublicense, or distribute the Program");
      Put_Line("except as expressly provided under this License.  Any attempt");
      Put_Line("otherwise to copy, modify, sublicense or distribute the Program is");
      Put_Line("void, and will automatically terminate your rights under this License.");
      Put_Line("However, parties who have received copies, or rights, from you under");
      Put_Line("this License will not have their licenses terminated so long as such");
      Put_Line("parties remain in full compliance.");
      Put_Line("");
      Put_Line("  5. You are not required to accept this License, since you have not");
      Put_Line("signed it.  However, nothing else grants you permission to modify or");
      Put_Line("distribute the Program or its derivative works.  These actions are");
      Put_Line("prohibited by law if you do not accept this License.  Therefore, by");
      Put_Line("modifying or distributing the Program (or any work based on the");
      Put_Line("Program), you indicate your acceptance of this License to do so, and");
      Put_Line("all its terms and conditions for copying, distributing or modifying");
      Put_Line("the Program or works based on it.");
      Put_Line("");
      Put_Line("  6. Each time you redistribute the Program (or any work based on the");
      Put_Line("Program), the recipient automatically receives a license from the");
      Put_Line("original licensor to copy, distribute or modify the Program subject to");
      Put_Line("these terms and conditions.  You may not impose any further");
      Put_Line("restrictions on the recipients' exercise of the rights granted herein.");
      Put_Line("You are not responsible for enforcing compliance by third parties to");
      Put_Line("this License.");
      Put_Line("");
      Put_Line("  7. If, as a consequence of a court judgment or allegation of patent");
      Put_Line("infringement or for any other reason (not limited to patent issues),");
      Put_Line("conditions are imposed on you (whether by court order, agreement or");
      Put_Line("otherwise) that contradict the conditions of this License, they do not");
      Put_Line("excuse you from the conditions of this License.  If you cannot");
      Put_Line("distribute so as to satisfy simultaneously your obligations under this");
      Put_Line("License and any other pertinent obligations, then as a consequence you");
      Put_Line("may not distribute the Program at all.  For example, if a patent");
      Put_Line("license would not permit royalty-free redistribution of the Program by");
      Put_Line("all those who receive copies directly or indirectly through you, then");
      Put_Line("the only way you could satisfy both it and this License would be to");
      Put_Line("refrain entirely from distribution of the Program.");
      Put_Line("");
      Put_Line("If any portion of this section is held invalid or unenforceable under");
      Put_Line("any particular circumstance, the balance of the section is intended to");
      Put_Line("apply and the section as a whole is intended to apply in other");
      Put_Line("circumstances.");
      Put_Line("");
      Put_Line("It is not the purpose of this section to induce you to infringe any");
      Put_Line("patents or other property right claims or to contest validity of any");
      Put_Line("such claims; this section has the sole purpose of protecting the");
      Put_Line("integrity of the free software distribution system, which is");
      Put_Line("implemented by public license practices.  Many people have made");
      Put_Line("generous contributions to the wide range of software distributed");
      Put_Line("through that system in reliance on consistent application of that");
      Put_Line("system; it is up to the author/donor to decide if he or she is willing");
      Put_Line("to distribute software through any other system and a licensee cannot");
      Put_Line("impose that choice.");
      Put_Line("");
      Put_Line("This section is intended to make thoroughly clear what is believed to");
      Put_Line("be a consequence of the rest of this License.");
      Put_Line("");
      Put_Line("  8. If the distribution and/or use of the Program is restricted in");
      Put_Line("certain countries either by patents or by copyrighted interfaces, the");
      Put_Line("original copyright holder who places the Program under this License");
      Put_Line("may add an explicit geographical distribution limitation excluding");
      Put_Line("those countries, so that distribution is permitted only in or among");
      Put_Line("countries not thus excluded.  In such case, this License incorporates");
      Put_Line("the limitation as if written in the body of this License.");
      Put_Line("");
      Put_Line("  9. The Free Software Foundation may publish revised and/or new versions");
      Put_Line("of the General Public License from time to time.  Such new versions will");
      Put_Line("be similar in spirit to the present version, but may differ in detail to");
      Put_Line("address new problems or concerns.");
      Put_Line("");
      Put_Line("Each version is given a distinguishing version number.  If the Program");
      Put_Line("specifies a version number of this License which applies to it and ""any");
      Put_Line("later version"", you have the option of following the terms and conditions");
      Put_Line("either of that version or of any later version published by the Free");
      Put_Line("Software Foundation.  If the Program does not specify a version number of");
      Put_Line("this License, you may choose any version ever published by the Free Software");
      Put_Line("Foundation.");
      Put_Line("");
      Put_Line("  10. If you wish to incorporate parts of the Program into other free");
      Put_Line("programs whose distribution conditions are different, write to the author");
      Put_Line("to ask for permission.  For software which is copyrighted by the Free");
      Put_Line("Software Foundation, write to the Free Software Foundation; we sometimes");
      Put_Line("make exceptions for this.  Our decision will be guided by the two goals");
      Put_Line("of preserving the free status of all derivatives of our free software and");
      Put_Line("of promoting the sharing and reuse of software generally.");
      Put_Line("");
      Put_Line("                            NO WARRANTY");
      Put_Line("");
      Put_Line("  11. BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY");
      Put_Line("FOR THE PROGRAM, TO THE EXTENT PERMITTED BY APPLICABLE LAW.  EXCEPT WHEN");
      Put_Line("OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES");
      Put_Line("PROVIDE THE PROGRAM ""AS IS"" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED");
      Put_Line("OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF");
      Put_Line("MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  THE ENTIRE RISK AS");
      Put_Line("TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU.  SHOULD THE");
      Put_Line("PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,");
      Put_Line("REPAIR OR CORRECTION.");
      Put_Line("");
      Put_Line("  12. IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING");
      Put_Line("WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MAY MODIFY AND/OR");
      Put_Line("REDISTRIBUTE THE PROGRAM AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES,");
      Put_Line("INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING");
      Put_Line("OUT OF THE USE OR INABILITY TO USE THE PROGRAM (INCLUDING BUT NOT LIMITED");
      Put_Line("TO LOSS OF DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY");
      Put_Line("YOU OR THIRD PARTIES OR A FAILURE OF THE PROGRAM TO OPERATE WITH ANY OTHER");
      Put_Line("PROGRAMS), EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE");
      Put_Line("POSSIBILITY OF SUCH DAMAGES.");
      Put_Line("");
      Put_Line("                     END OF TERMS AND CONDITIONS");
      Put_Line("");
      Put_Line("            How to Apply These Terms to Your New Programs");
      Put_Line("");
      Put_Line("  If you develop a new program, and you want it to be of the greatest");
      Put_Line("possible use to the public, the best way to achieve this is to make it");
      Put_Line("free software which everyone can redistribute and change under these terms.");
      Put_Line("");
      Put_Line("  To do so, attach the following notices to the program.  It is safest");
      Put_Line("to attach them to the start of each source file to most effectively");
      Put_Line("convey the exclusion of warranty; and each file should have at least");
      Put_Line("the ""copyright"" line and a pointer to where the full notice is found.");
      Put_Line("");
      Put_Line("    <one line to give the program's name and a brief idea of what it does.>");
      Put_Line("    Copyright (C) <year>  <name of author>");
      Put_Line("");
      Put_Line("    This program is free software; you can redistribute it and/or modify");
      Put_Line("    it under the terms of the GNU General Public License as published by");
      Put_Line("    the Free Software Foundation; either version 2 of the License, or");
      Put_Line("    (at your option) any later version.");
      Put_Line("");
      Put_Line("    This program is distributed in the hope that it will be useful,");
      Put_Line("    but WITHOUT ANY WARRANTY; without even the implied warranty of");
      Put_Line("    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the");
      Put_Line("    GNU General Public License for more details.");
      Put_Line("");
      Put_Line("    You should have received a copy of the GNU General Public License along");
      Put_Line("    with this program; if not, write to the Free Software Foundation, Inc.,");
      Put_Line("    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.");
      Put_Line("");
      Put_Line("Also add information on how to contact you by electronic and paper mail.");
      Put_Line("");
      Put_Line("If the program is interactive, make it output a short notice like this");
      Put_Line("when it starts in an interactive mode:");
      Put_Line("");
      Put_Line("    Gnomovision version 69, Copyright (C) year name of author");
      Put_Line("    Gnomovision comes with ABSOLUTELY NO WARRANTY; for details type `show w'.");
      Put_Line("    This is free software, and you are welcome to redistribute it");
      Put_Line("    under certain conditions; type `show c' for details.");
      Put_Line("");
      Put_Line("The hypothetical commands `show w' and `show c' should show the appropriate");
      Put_Line("parts of the General Public License.  Of course, the commands you use may");
      Put_Line("be called something other than `show w' and `show c'; they could even be");
      Put_Line("mouse-clicks or menu items--whatever suits your program.");
      Put_Line("");
      Put_Line("You should also get your employer (if you work as a programmer) or your");
      Put_Line("school, if any, to sign a ""copyright disclaimer"" for the program, if");
      Put_Line("necessary.  Here is a sample; alter the names:");
      Put_Line("");
      Put_Line("  Yoyodyne, Inc., hereby disclaims all copyright interest in the program");
      Put_Line("  `Gnomovision' (which makes passes at compilers) written by James Hacker.");
      Put_Line("");
      Put_Line("  <signature of Ty Coon>, 1 April 1989");
      Put_Line("  Ty Coon, President of Vice");
      Put_Line("");
      Put_Line("This General Public License does not permit incorporating your program into");
      Put_Line("proprietary programs.  If your program is a subroutine library, you may");
      Put_Line("consider it more useful to permit linking proprietary applications with the");
      Put_Line("library.  If this is what you want to do, use the GNU Lesser General");
      Put_Line("Public License instead of this License.");
   end ShowGPL;

begin

   if Argument_Count > 3 then
      Set_Exit_Status(Failure);
      Put_Line(Standard_Error, "Wrong number of arguments.");
      Put_Line(Standard_Error, "");
      Usage;
      return;
   end if;

   declare
      i: Integer := 1;
   begin

      while i <= Argument_Count loop

         if Argument(i) = "-o" then
            if i+1 <= Argument_Count then
               i := i + 1;
               Options.Dir := To_Unbounded_String(Argument(i));
            else
               Set_Exit_Status(Failure);
               Put_Line(Standard_Error, "Missing directory name.");
               Put_Line(Standard_Error, "");
               Usage;
               return;
            end if;
         elsif Argument(i) = "-f" then
            Options.Force := True;

         elsif Argument(i) = "-l" then
            ShowGPL;
            Set_Exit_Status(Success);
            return;

         else
            Set_Exit_Status(Failure);
            Put_Line(Standard_Error, "Unknown option: " & Argument(i));
            Put_Line(Standard_Error, "");
            Usage;
            return;

         end if;

         i := i + 1;
      end loop;
   end;

   -- create output directory
   if To_String(Options.Dir) /= "" then
      declare
      begin
         if Ada.Directories.Exists(To_String(Options.Dir)) then
            if Options.Force then
               Ada.Directories.Delete_Tree(To_String(Options.Dir));
            else
               raise Ada.Directories.Use_Error;
            end if;
         end if;

         Ada.Directories.Create_Directory(To_String(Options.Dir));

      exception
         when Ex: Ada.Directories.Use_Error |
           Ada.Directories.Name_Error =>
            Put_Line(Standard_Error, "Unable to create directory: " & To_String(Options.Dir));
            Set_Exit_Status(Failure);
            return;
      end;
   end if;

   Asis.Implementation.Initialize     (Initialization_Parameters);

   Asis.Ada_Environments.Associate
     (The_Context => My_Context,
      Name        => My_Context_Name,
      Parameters  => My_Context_Parameters);

   Asis.Ada_Environments.Open         (My_Context);

   Context_Processing.Process_Context (The_Context => My_Context,
                                       Trace       => True);

   Asis.Ada_Environments.Close        (My_Context);
   Asis.Ada_Environments.Dissociate   (My_Context);
   Asis.Implementation.Finalize       (Finalization_Parameters);

exception

   when Ex : Asis.Exceptions.ASIS_Inappropriate_Context          |
             Asis.Exceptions.ASIS_Inappropriate_Container        |
             Asis.Exceptions.ASIS_Inappropriate_Compilation_Unit |
             Asis.Exceptions.ASIS_Inappropriate_Element          |
             Asis.Exceptions.ASIS_Inappropriate_Line             |
             Asis.Exceptions.ASIS_Inappropriate_Line_Number      |
             Asis.Exceptions.ASIS_Failed                         =>

      Ada.Wide_Text_IO.Set_Output(Ada.Wide_Text_IO.Standard_Error);

      Ada.Wide_Text_IO.Put ("ASIS exception (");
      Ada.Wide_Text_IO.Put (Ada.Characters.Handling.To_Wide_String (
              Ada.Exceptions.Exception_Name (Ex)));
      Ada.Wide_Text_IO.Put (") is raised");
      Ada.Wide_Text_IO.New_Line;

      Ada.Wide_Text_IO.Put ("ASIS Error Status is ");
      Ada.Wide_Text_IO.Put
        (Asis.Errors.Error_Kinds'Wide_Image (Asis.Implementation.Status));
      Ada.Wide_Text_IO.New_Line;

      Ada.Wide_Text_IO.Put ("ASIS Diagnosis is ");
      Ada.Wide_Text_IO.New_Line;
      Ada.Wide_Text_IO.Put (Asis.Implementation.Diagnosis);
      Ada.Wide_Text_IO.New_Line;

      Asis.Implementation.Set_Status;
      Ada.Wide_Text_IO.Set_Output(Ada.Wide_Text_IO.Standard_Output);

   when Ex : others =>

      Ada.Wide_Text_IO.Set_Output(Ada.Wide_Text_IO.Standard_Error);

      Ada.Wide_Text_IO.Put (Ada.Characters.Handling.To_Wide_String (
              Ada.Exceptions.Exception_Name (Ex)));
      Ada.Wide_Text_IO.Put (" is raised (");
      Ada.Wide_Text_IO.Put (Ada.Characters.Handling.To_Wide_String (
              Ada.Exceptions.Exception_Information (Ex)));
      Ada.Wide_Text_IO.Put (")");
      Ada.Wide_Text_IO.New_Line;

      Ada.Wide_Text_IO.Set_Output(Ada.Wide_Text_IO.Standard_Output);

end ast2dot;
