---------------------------------------------------------------------------
--                                                                       --
--  ast2dot is an ASIS application that outputs the AST in dot format.   --
--                          http://cfg.w3x.org                           --
--                                                                       --
---------------------------------------------------------------------------
--                                                                       --
--        Copyright (c) 2006,2009 Raul Fechete, Georg Kienesberger       --
--                                                                       --
---------------------------------------------------------------------------
-- This file is part of ast2dot.                                         --
--                                                                       --
-- ast2dot is free software; you can redistribute it and/or modify       --
-- it under the terms of the GNU General Public License as published by  --
-- the Free Software Foundation; either version 2 of the License, or     --
-- (at your option) any later version.                                   --
--                                                                       --
-- ast2dot is distributed in the hope that it will be useful,            --
-- but WITHOUT ANY WARRANTY; without even the implied warranty of        --
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         --
-- GNU General Public License for more details.                          --
--                                                                       --
-- You should have received a copy of the GNU General Public License     --
-- along with ast2dot; if not, write to the Free Software                --
-- Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 --
-- USA                                                                   --
--                                                                       --
---------------------------------------------------------------------------

with Ada.Exceptions;

package body Transformation_Control is

   -- Tree level control functions

   procedure ZeroLevel is
   begin
      Tree_Level := 0;
      Last_Tree_Level := -1;
      Empty(LevList);
   end ZeroLevel;

   procedure IncLevel is
   begin
      Last_Tree_Level := Tree_Level;
      Tree_Level := Tree_Level + 1;
      Set(LevList, Tree_Level, GetNextId);
   end IncLevel;

   procedure DecLevel is
   begin
      if Tree_Level > 0 then
         Last_Tree_Level := Tree_Level;
         Tree_Level := Tree_Level - 1;
      else
         raise Underflow;
      end if;
   end DecLevel;

   function GetLevel return Natural is
   begin
      return Tree_Level;
   end GetLevel;

   function GetLastLevel return Natural is
   begin
      if Last_Tree_Level > -1 then
         return Last_Tree_Level;
      else
         raise Unset;
      end if;
   end GetLastLevel;

   procedure ZeroId is
   begin
      Id := 0;
      Empty(LevList);
   end ZeroId;

   function GetId return Natural is
   begin
      return Id;
   end GetId;

   function GetNextId return Natural is
   begin
      Id := Id + 1;

      if Id = Natural'Last then
         Ada.Exceptions.Raise_Exception(Node_Overflow'Identity, "Maximum node count reached.");
      end if;

      return Id;
   end GetNextId;

end Transformation_Control;
